# ======================================================================
# Define options
# ======================================================================
set val(chan)           Channel/WirelessChannel    ;# Channel Type
set val(prop)           Propagation/TwoRayGround   ;# radio-propagation model
set val(netif)          Phy/WirelessPhy/802_15_4
set val(mac)            Mac/802_15_4		;
#set val(mac)		inaceswrapper	;
set val(ifq)            Queue/DropTail/PriQueue    ;# interface queue type
set val(ll)             LL                         ;# link layer type
set val(ant)            Antenna/OmniAntenna        ;# antenna model
set val(ifqlen)         50                         ;# max packet in ifq
set val(nn)             3                         ;# number of mobilenodes
set val(rp)             AODV                       ;# AODV Routing / DumbAgent
set val(x)		20
set val(y)		20
set beaconOrder		8 ;
set superFrameOrder		8 ;
set val(nam)		wpan_demo4.nam
set val(traffic)	mix                        ;# mix/cbr/poisson/ftp
#Duration of the simulation
set stopTime            300	;# in seconds 

#set debug 1;     
#read command line arguments
 for {set i 0} {$i < [llength $argv]}  {incr i}  {
set val(nn) [lindex $argv 0]
	set  beaconOrder [lindex $argv 1]
set  superFrameOrder [lindex $argv 2]
set stopTime  [lindex $argv 3]
 }


#Set Frame Types

set Beacon 0
set Data 1
set Ack 2
set Mac 3
set AssociationRequest 1
set AssociationResponse 2
set DisassociationNotification 3
set DataRequest 4
set PANiDconflictNotification 5
set OrphanNotification 6
set BeaconRequest 7
set CoordinatorRealignment 8
set GTSrequest 9

#Set Scenarios for Temporal or Energetical Account
set ALL 0
set SBFL 1
set MBFL 2
set SYNCLOSS 3
set ORPH 4
set COORDREAL 5
set PANIDCONFLICT 6
set GTSREQUEST 7


# Initialize Global Variables
set ns_		[new Simulator]
set tracefd     [open ./wpan_demo4.tr w]
$ns_ trace-all $tracefd




$ns_ trace-all $tracefd
if { "$val(nam)" == "wpan_demo4.nam" } {
        set nf     [open ./$val(nam) w]
        $ns_ namtrace-all-wireless $nf $val(x) $val(y)
}

$ns_ puts-nam-traceall {# nam4wpan #}		;# inform nam that this is a trace file for wpan (special handling needed)

#trace nam to see graphical interface
#set nf [open $val(nam) w]
#$ns_ namtrace-all-wireless $nf $val(x) $val(y)

# For model 'TwoRayGround'
set dist(5m)  7.69113e-06
set dist(9m)  2.37381e-06
set dist(10m) 1.92278e-06
set dist(11m) 1.58908e-06
set dist(12m) 1.33527e-06
set dist(13m) 1.13774e-06
set dist(14m) 9.81011e-07
set dist(15m) 8.54570e-07
set dist(16m) 7.51087e-07
set dist(20m) 4.80696e-07
set dist(25m) 3.07645e-07
set dist(30m) 2.13643e-07
set dist(35m) 1.56962e-07
set dist(40m) 1.20174e-07
Phy/WirelessPhy set CSThresh_ $dist(15m)
Phy/WirelessPhy set RXThresh_ $dist(15m)

# set up topography object
set topo       [new Topography]
$topo load_flatgrid $val(x) $val(y)

# Create God
set god_ [create-god $val(nn)]

set chan_1_ [new $val(chan)]

#Verbose mode On
Mac/802_15_4 wpanCmd verbose on
#nam animations definitions

Mac/802_15_4 wpanNam namStatus on
Mac/802_15_4 wpanNam emHandling on
Mac/802_15_4 wpanNam CoorClr blue
Mac/802_15_4 wpanNam DevClr gold
Mac/802_15_4 wpanNam ColFlashClr red # default = gold
Mac/802_15_4 wpanNam NodeFailClr grey # default = grey
Mac/802_15_4 wpanNam FlowClr -p AODV -c tomato
Mac/802_15_4 wpanNam FlowClr -p ARP -c green 

# configure node

$ns_ node-config -adhocRouting $val(rp) \
		-llType $val(ll) \
		-macType $val(mac) \
		-ifqType $val(ifq) \
		-ifqLen $val(ifqlen) \
		-antType $val(ant) \
		-propType $val(prop) \
		-phyType $val(netif) \
		-topoInstance $topo \
		-agentTrace OFF \
		-routerTrace OFF \
		-macTrace ON \
		-movementTrace OFF \
                -energyModel "EnergyModel" \
                -initialEnergy 10 \
		-idlePower 0.0 \
                -rxPower 0.03 \
                -txPower 0.03 \
		-channel $chan_1_
	#	-IncomingErrProc UniformErr \
#reduzir a energia cada x que executa uma accao
#-rxPower 0.175 \
#-txPower 0.175 \
#-sensePower 0.00000175; \
#-idlePower 0.0 \
#-initialEnergy 0.5

#rxPower .175 indicates 175mW con-
#sumed for receiving a packet of arbitrary size,

#txPower .175 indicates 175mW con-
#sumed for transmitting a packet of arbitrary
#size,

#sensePower .00000175
#indicates
#1.75μW consumed for receiving a PHENOM
#broadcast packet,

#initialEnergy 5 indicates a total energy
#reserve of 5J .

   # for {set i 0}{$i < val(nn)}{incr i} {
   #     set n($i) [$ns node]
   # }



	proc UniformErr {} {

#error model
set em [new ErrorModel/SelectErrorModel]

$em set enable_ 1
$em set markecn_ true
$em set rate_ 1
$em set bandwidth_ 2Mb
$em set pkt_type_ 35
$em set drop_cycle_ 10
$em set drop_offset_ 1
$em set debug_ true
$em set delay_pkt_ false
$em set delay_ 0
return $em
	}


for {set i 0} {$i < $val(nn) } {incr i} {
		set node_($i) [$ns_ node]		 ;# Create nodes
		$node_($i) random-motion 0		;# disable random motion
	}

# defines the node size in nam
for {set i 0} {$i < $val(nn)} {incr i} {
 $ns_ initial_node_pos $node_($i) 2
}
# define the distance between the nodes at an uniform distance Auto or from file
#set rng [new RNG]
 #  $rng seed 0
##set ptr [open "coord1" r]
#for { set i 0 } {$i < $val(nn)} { incr i } {
##	gets $ptr line
##	set x $line
#	set x [$rng uniform 0.0 $val(x)]
#	set y [$rng uniform 0.0 $val(y)]
#	$node_($i) set X_ $x
##	gets $ptr line
##	set y $line
#	$node_($i) set Y_ $y
#	$node_($i) set Z_ 0.0
#	$node_($i) radius [$rng uniform 200.0 250.0]
#}
# define the distance between the nodes at an uniform distance Auto or from file
set rng [new RNG]
   $rng seed 0
#set ptr [open "coord1" r]
for { set i 0 } {$i < $val(nn)} { incr i } {
#	gets $ptr line
#	set x $line
	set x [$rng uniform 0.0 $val(x)]
	set y [$rng uniform 0.0 $val(y)]
	$node_($i) set X_ $x
#	gets $ptr line
#	set y $line
	$node_($i) set Y_ $y
	$node_($i) set Z_ 0.0
	$node_($i) radius [$rng uniform 200.0 250.0]
}
#manualy define the position of nodes
$node_(0) set X_ 6
$node_(0) set Y_ 6

$node_(0) set Z_ 0.0

$node_(1) set X_ 4
$node_(1) set Y_ 12
$node_(1) set Z_ 0.0

$node_(2) set X_ 14
$node_(2) set Y_ 14
$node_(2) set Z_ 0.0
#manualy define the position of nodes


#$node_(3) set X_ 10
#$node_(3) set Y_ 10
#$node_(3) set Z_ 0.0

#close $ptr

#TRAFEGO..BEGIN
set udp_(0) [new Agent/UDP]
$ns_ attach-agent $node_(1) $udp_(0)
set null_(0) [new Agent/Null]
$ns_ attach-agent $node_(0) $null_(0)
set cbr_(0) [new Application/Traffic/CBR]
$cbr_(0) set rate_ 1Mb
$cbr_(0) set packetSize_ 100
$cbr_(0) set interval_ 0.25
#$cbr_(0) set random_ 1
#$cbr_(0) set maxpkts_ 10000
$cbr_(0) attach-agent $udp_(0)
$ns_ connect $udp_(0) $null_(0)
$ns_ at 35.0 "$cbr_(0) start"
$ns_ at 70.0 "$cbr_(0) stop"

#TRAFEGO..END 
#comandos da camada SSCS WPAN
# --- $node sscs startPANCoord	<txBeacon = 1> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startDevice <isFFD = 1> <assoPermit = 1> <txBeacon = 0> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startCTPANCoord <txBeacon = 1> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startCTDevice <isFFD = 1> <assoPermit = 1> <txBeacon = 0> <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs startBeacon <beaconOrder = 3> <SuperframeOrder = 3>
# --- $node sscs stopBeacon
#--Added Functions----------------------------------
# --- $node sscs txOption GTS <On/Off>
# --- $node sscs panConflict <0 = off/ 1= on>

#comandos do Fault Injector e Temporal/Energy account
# --- $node sscs enableEnergyAccount <scenario=0> <start= 0.0> <end=0.0>
# --- $node sscs enableTemporalAccount <scenario=0>
# --- $node sscs faultInjection <frame=0> <ntimes= 0> <node=0>
# --- $node sscs randomFaultInjection <stop=simulationTime> <frame=0> <ntimes= 0> <node=0>

# frame:
#{0="M_BEACON",1="M_CM1_AssoReq",2="M_CM2_AssoRsp",3="M_CM3_DisaNtf",4="M_CM4_DataReq",5="M_CM5_PidCNtf",6="M_CM6_OrphNtf",7="M_CM7_Bcn-Req",8="M_CM8_CoorRal",9="M_CM9_GTS-Req",10="M_ACK",11="M_UNKN"}

#Schedule of Events
$ns_ at 0.0	"$node_(0) NodeLabel \"PAN Coor\""
$ns_ at 0.0	"$node_(0) sscs startPANCoord 1 $beaconOrder $superFrameOrder";
$ns_ at 70.0	"$node_(1) sscs startDevice 0 0 0 $beaconOrder $superFrameOrder"
$ns_ at 70.0	"$node_(2) sscs startDevice 0 0 0 $beaconOrder $superFrameOrder"
#$ns_ at 20.0	"$node_(3) sscs startDevice 0 1" The devices have to be started manually
#$ns_ at 20.0	"$node_(4) sscs startDevice 0 1"
#$ns_ at 20.0	"$node_(5) sscs startDevice 0 1"
#$ns_ at 20.0	"$node_(6) sscs startDevice 0 1"
#$ns_ at 20.0	"$node_(7) sscs startDevice 0 1"
#$ns_ at 65.0 	"$node_(0) sscs startBeacon $beaconOrder $superFrameOrder"
$ns_ at 140.0	"$node_(1) sscs enableTemporalAccount $ALL";
#$ns_ at 140.0	"$node_(0) sscs enableTemporalAccount $ALL";
$ns_ at 30.0	"$node_(1) sscs txOption GTS On"
$ns_ at 30.0	"$node_(2) sscs txOption GTS On"
#$ns_ at 40.0	"$node_(1) sscs enableEnergyAccount 43.305792 47.237952";
$ns_ at 140.0	"$node_(1) sscs enableEnergyAccount 0 0 0";
#$ns_ at 40.0 "puts \"NS faultInjection...\n\""
#$ns_ at 30.0	"$node_(1) sscs randomFaultInjection $stopTime 0 8"
#$ns_ at 40.0	"$node_(1) sscs txOption GTS Off"
$ns_ at 140.0	"$node_(1) sscs faultInjection $Beacon 5"
$ns_ at 160.0	"$node_(0) sscs faultInjection 6 1"
#$ns_ at 40\.0	"$node_(2) sscs faultInjection 0 8"
#ns_ at 80.0	"$node_(1) sscs disableFaultInjection"
#$ns_ at 62.0	"$node_(1) sscs disableFaultInjection"
#$ns_ at 43.0 "puts \"NS faultInjection...\n\""
#$ns_ at 40\.0	"$node_(2) sscs faultInjection 0"
#$ns_ at 50.0	"$node_(1) sscs startSync"
#$ns_ at 50.0	"$node_(2) sscs startSync"
$ns_ at $stopTime "stop"
$ns_ at $stopTime "puts \"NS EXITING...\n\""
$ns_ at $stopTime "$ns_ halt"
proc stop {} {
    global ns_ tracefd appTime1 val env
    $ns_ flush-trace
    close $tracefd
#exec nam $val(nam) &
exit 0
  
}

puts "\nStarting Simulation..."
$ns_ run

