/*
 * temporalAccount.cc
 * Represents a Temporal and Energetic Account module
 * to record the time and energy of the frame receptions events
 *
 *  Created on: Dec 27, 2012
 *      Author: aguerreiro
 */


#include "temporalAccount.h"

temporalAccount ta;
energyAccount ea;
// --0x01:	Association request (Figures 48 -> 49)
// --0x02:	Association response (Figure 50;Table 68)
// --0x03:	Disassociation notification (Figure 51;Table 69)
// --0x04:	Data request (Figure 52)
// --0x05:	PAN ID conflict notification (Figure 53)
// --0x06:	Orphan notification (Figure 54)
// --0x07:	Beacon request (Figure 55)
// --0x08:	Coordinator realignment (Figure 56
// --0x09:	GTS request (Figures 57 -> 58)
int counterT=0;
int counterE=0;
char *scenarios[] = {"ALL","SBFL", "MBFL", "SYNCLOSS","ORPH","COORDREAL","PANIDCONFLICT","GTSREQUEST"};
void eventTime(UINT_8 numLostBeacons,int index_,UINT_8 MSDU_CmdType ,	UINT_16	MHR_FrmCtrl,bool isPANCoor){

	if (numLostBeacons == 1 && ta.nodeId == index_ && ta.time1 ==0.0)
	{
		ta.time1 = CURRENT_TIME;
	}
	if (numLostBeacons == 2 && ta.nodeId == index_ && ta.time2 ==0.0)
	{
		ta.time2 = CURRENT_TIME;
		ta.time_sbfl = ta.time2 - ta.time1;
		if(ta.scenario==1){
			fprintf(stdout,"\n 2nd beacon loss occurred: Simulation time - %f seconds at node[%d] \n",ta.time2,index_);
			fprintf(stdout,"\n Single beacon frame loss: %f seconds at node[%d] \n",ta.time_sbfl,index_);
		}
	}
	if (numLostBeacons == 3 && ta.time3 ==0.0)
	{
		ta.time3 = CURRENT_TIME;
	}
	if (numLostBeacons == aMaxLostBeacons && ta.time4 ==0.0)
	{

		ta.time4 = CURRENT_TIME;
		ta.time_mbfl = ta.time4 - ta.time1;



		if(ta.scenario==2)
			fprintf(stdout,"\n Multiple beacon frame loss: %f seconds at node[%d] \n",ta.time_mbfl,index_);
		if(ta.scenario==3)
			fprintf(stdout,"\n Synchronization loss: %f seconds at node[%d] \n",ta.time_mbfl,index_);
	}
	if(MSDU_CmdType==5  && ta.nodeId == index_ && ta.time_panidconflict == 0.0 ){
		ta.time_panidconflict=CURRENT_TIME-ta.time1;
		if(ta.scenario==6||ta.scenario==0)
			fprintf(stdout,"\n PAN Conflict Detection occurred: Simulation time - %f seconds: duration:%f seconds at node[%d] \n",CURRENT_TIME,ta.time_panidconflict,index_);
	}
	//Orphan Node - Observado nos nós
	if(MSDU_CmdType==6  && ta.nodeId == index_&& ta.time_ophNode == 0.0 && !isPANCoor){
		ta.time_ophNode=CURRENT_TIME;
	}
	//Orphan Node - Observado no Cordenador
	if(MSDU_CmdType==6  && ta.time_ophCoord == 0.0 && isPANCoor){
		ta.time_ophCoord=CURRENT_TIME;
	}
	//Coordinator Realignment  Observado no Coordenador
	if(MSDU_CmdType==8  && ta.time_cordrealignCoord == 0.0 && isPANCoor){
		ta.time_cordrealignCoord=CURRENT_TIME-ta.time_ophCoord;
		if(ta.scenario==5||ta.scenario==0)
			fprintf(stdout,"\n Coordinator Realignment occurred: Simulation time - %f seconds: duration:%f seconds at node[%d] \n",CURRENT_TIME,ta.time_cordrealignCoord,index_);
	}///Coordinator Realignment  Observado no Nó
	if(MSDU_CmdType==8  && ta.nodeId == index_ && ta.time_cordrealignNode == 0.0 && !isPANCoor){
		ta.time_cordrealignNode=(CURRENT_TIME-ta.time_ophNode) + ta.time_mbfl;
		if(ta.scenario==5||ta.scenario==0)
			fprintf(stdout,"\n Orphan Notification occurred: Simulation time - %f seconds: duration:%f seconds at node[%d] \n",CURRENT_TIME,ta.time_cordrealignNode,index_);
	}
	//GTS REQUEST
	if(MSDU_CmdType==9  && ta.nodeId == index_ && ta.time_gtsReq == 0.0 ){
		if(ta.scenario==7||ta.scenario==0)
			ta.time_gtsReq=CURRENT_TIME;


	}
	//16384 = ACK
	if(MHR_FrmCtrl==16384  && ta.nodeId == index_ && ta.time_gtsAck == 0.0 && ta.time_gtsReq > 0.0){
		ta.time_gtsAck=CURRENT_TIME;
		ta.time_gtsReq=ta.time_gtsAck-ta.time_gtsReq;
		if(ta.scenario==7||ta.scenario==0)
			fprintf(stdout,"\n GTS Request occurred: Simulation time - %f seconds: duration:%f seconds at node[%d] \n",CURRENT_TIME,ta.time_gtsReq,index_);
	}

	if(numLostBeacons == aMaxLostBeacons && ta.time4 !=0.0 && ta.scenario==0 && counterT==0){
		fprintf(stdout,"\n Duration of the measured network inaccessibility scenarios at node [%d] \n" ,index_);
		fprintf(stdout,"\n 1st beacon loss occurred: Simulation time - %f seconds at node[%d] \n",ta.time1,index_);
		fprintf(stdout,"\n 2nd beacon loss occurred: Simulation time - %f seconds at node[%d] \n",ta.time2,index_);
		fprintf(stdout,"\n 3rd beacon loss occurred: Simulation time - %f seconds at node[%d] \n",ta.time3,index_);
		fprintf(stdout,"\n 4rt beacon loss occurred: Simulation time - %f seconds at node[%d] \n",ta.time4,index_);
		fprintf(stdout,"\n Single beacon frame loss: %f seconds at node[%d] \n",ta.time_sbfl,index_);
		fprintf(stdout,"\n Multiple beacon frame loss: %f seconds at node[%d] \n",ta.time_mbfl,index_);
		fprintf(stdout,"\n Synchronization loss: %f seconds at node[%d] \n",ta.time_mbfl,index_);
		counterT++;
	}


}

void enabletempAccount(temporalAccount tx){
	ta=tx;
}

void eventEnergy(UINT_8 numLostBeacons,int index_,double energy,UINT_8 MSDU_CmdType,	UINT_16	MHR_FrmCtrl,bool isPANCoor ){
	if (numLostBeacons == 1 && ea.energy1 ==0.0 && ea.nodeId == index_)
	{
		ea.energy1 = energy;
	}
	if (numLostBeacons == 2  && ea.energy2 ==0.0 && ea.nodeId == index_)
	{
		ea.energy2 =energy;
		ea.energy_sbfl = ea.energy1 - ea.energy2;
		if(ea.scenario==1){
			fprintf(stdout,"\n 2nd beacon loss occurred: Simulation Energy - %f at node[%d] \n",ea.energy2,index_);
			fprintf(stdout,"\n Single beacon frame loss: %f energy at node[%d] \n",ea.energy_sbfl,index_);
		}
	}
	if (numLostBeacons == 3 && ea.energy3 ==0.0  && ea.nodeId == index_)
	{
		ea.energy3 = energy;
	}
	if (numLostBeacons == aMaxLostBeacons && ea.energy4 ==0.0  && ea.nodeId == index_)
	{
		ea.energy4 = energy;
		ea.energy_mbfl = ea.energy1 - ea.energy4;

		if(ta.scenario==2)
			fprintf(stdout,"\n Multiple beacon frame loss: %f energy at node[%d] \n",ea.energy_mbfl,index_);
		if(ta.scenario==3)
			fprintf(stdout,"\n Synchronization loss: %f energy at node[%d] \n",ea.energy_mbfl,index_);
	}
	if(ea.diff){
		if((fabs(ea.start-CURRENT_TIME)<0.0000005)&&ea.nodeId==index_){
			ea.energy1=energy;

		}
		if((fabs(ea.stop-CURRENT_TIME)<0.0000005)&&ea.nodeId==index_){
			ea.energy2=energy;
			ea.energy_sbfl = ea.energy1 - ea.energy2;
			if(ea.scenario==6||ea.scenario==0)
				fprintf(stdout,"\n Energy consumed in interval [%f,%f] was: %f  at node[%d] \n",ea.start,ea.stop,ea.energy_sbfl,index_);
		}

	}
	if(MSDU_CmdType==5  && ea.nodeId == index_ && ea.energy_panidconflict == 0.0 ){
		ea.energy_panidconflict=ea.energy1-energy;
		if(ea.scenario==6||ea.scenario==0)
			fprintf(stdout,"\n PAN Conflict Detection occurred:  %f energy at node[%d] \n",ea.energy_panidconflict,index_);
	}
	//Orphan Node - Observado nos nós
	if(MSDU_CmdType==6  && ea.nodeId == index_&& ea.energy_ophNode == 0.0 && !isPANCoor){
		ea.energy_ophNode=energy;

	}
	//Orphan Node - Observado no Cordenador
	if(MSDU_CmdType==6  && ea.energy_ophCoord == 0.0 && isPANCoor){
		ea.energy_ophCoord=energy;

	}
	//Coordinator Realignment  Observado no Coordenador
	if(MSDU_CmdType==8  && ea.energy_cordrealignCoord == 0.0 && isPANCoor){
		ea.energy_cordrealignCoord=ea.energy_ophCoord-energy;
		if(ea.scenario==5||ea.scenario==0)
			fprintf(stdout,"\n Coordinator Realignment occurred: %f energy at node[%d] \n",ea.energy_cordrealignCoord,index_);
	}///Coordinator Realignment  Observado no Nó
	if(MSDU_CmdType==8  && ea.nodeId == index_ && ea.energy_cordrealignNode == 0.0 && !isPANCoor){
		ea.energy_cordrealignNode =(ea.energy_ophNode-energy)+ea.energy_mbfl;
		if(ea.scenario==5||ea.scenario==0)
			fprintf(stdout,"\n Orphan Notification occurred: %f energy at node[%d] \n",ea.energy_cordrealignNode,index_);
	}
	//GTS REQUEST
	if(MSDU_CmdType==9  && ea.nodeId == index_ && ea.energy_gtsReq == 0.0 ){
		if(ea.scenario==7||ea.scenario==0)
			ea.energy_gtsReq =energy;


	}
	//16384 = ACK
	if(MHR_FrmCtrl==16384  && ea.nodeId == index_ && ea.energy_gtsAck == 0.0 && ea.energy_gtsReq > 0.0){
		ea.energy_gtsAck=energy;
		ea.energy_gtsReq=ea.energy_gtsReq-ea.energy_gtsAck;
		if(ea.scenario==7||ea.scenario==0)
			fprintf(stdout,"\n GTS Request occurred: %f energy at node[%d] \n",ea.energy_gtsReq,index_);
	}

	if (numLostBeacons == aMaxLostBeacons && ea.energy4 !=0.0 && ea.scenario==0 && counterE==0)
	{
		fprintf(stdout,"\n Analysis of the measured  Energy during network inaccessibility scenarios at node [%d] \n" ,index_);
		fprintf(stdout,"\n 1st beacon loss occurred:  Energy - %f at %f seconds at node[%d] \n",ea.energy1,ta.time1,index_);
		fprintf(stdout,"\n 2nd beacon loss occurred:  Energy - %f at %f seconds at node[%d] \n",ea.energy2,ta.time2,index_);
		fprintf(stdout,"\n 3rd beacon loss occurred:  Energy - %f at %f seconds at node[%d] \n",ea.energy3,ta.time3,index_);
		fprintf(stdout,"\n 4rt beacon loss occurred:  Energy - %f at %f seconds at node[%d] \n",ea.energy4,ta.time4,index_);
		fprintf(stdout,"\n Single beacon frame loss: %f energy at node[%d] \n",ea.energy_sbfl,index_);
		fprintf(stdout,"\n Multiple beacon frame loss: %f energy at node[%d] \n",ea.energy_mbfl,index_);
		fprintf(stdout,"\n Synchronization loss: %f energy at node[%d] \n",ea.energy_mbfl,index_);
		counterE++;
	}
}

void enableEnergyAccount(energyAccount ex){
	ea=ex;
}

